#include "stdint.h"
#include "s3c44b0x.h"
#include "font.h"

void delay_ms(uint32_t delay)
{
    /* Delay hand tuned for use with 66.428MHz clock
       (PLLCON = 0x55051, PLL reference of 10MHz).
       Assumes code is running from cached memory.
       fixme: use a hardware timer instead.
    */

    uint32_t ms;
    uint32_t i;
    for (ms=0; ms<delay; ms++)
        for (i=0; i<13288; i++)
            ;
}

void cpu_init()
{
    /* Set clock rate to 66.428MHz, just in case the JB has
       it set wrong
    */
    *pllcon=0x55051;

    /* Be sure we're in cacheable memory
       Can the framebuffer be in cache too?
    */
    *ncachbe0=0x80002000;
    *ncachbe1=0x0;
}

uint32_t rand()
{
    /* This is a really bad linear congruential generator */
    static uint32_t r;
    r=(1664525L*r)+1013904223L;
    return r;
}

uint32_t *framebuffer_start()
{
    /* Return with starting address of LCD framebuffer */
    return (uint32_t*)(((*lcdsaddr1) & 0x07ffffff) << 1);
}

void clear_lcd()
{
    uint32_t i;
    uint32_t *addr=framebuffer_start();

    /* fill entire framebuffer, 32 bits at a whack */
    for (i=0; i<240*160; i+=4)
    {
        *addr=0;
        ++addr;
    }
}

void set_pixel(uint32_t x, uint32_t y, uint8_t color)
{
    uint8_t *addr=(uint8_t*)framebuffer_start();
    addr+=x;
    addr+=(240*y);
    *addr=color;
}

void lcd_init()
{
    /* Set ENVID (bit 0 of lcdcon1) to 0 to disable panel */
    *lcdcon1=0x10C0DF40;    /* LCDCON1  (LCD Control 1) */

    /* Configure the LCD panel, using settings borrowed from
       the built in self test code.
       Looks like it's one byte/pixel, using the palette LUT registers.
    */
    *lcdcon2=0x0141649F;    /* LCDCON2  (LCD Control 2) */
    *lcdcon3=0x00000000;    /* LCDCON3  (LCD Control 3) */
    *lcdsaddr1=0x1E0F288A;  /* LCDSADDR1  (Frame Upper Buffer Start Address 1) */
    *lcdsaddr2=0x21AF738A;  /* LCDSADDR2  (Frame Lower Buffer Start Address 2) */
    *lcdsaddr3=0x00000078;  /* LCDSADDR3  (Virtual Screen Address) */
    *redlut=0xFDB96420;     /* REDLUT  (RED Lookup Table) */
    *greenlut=0xFDB96420;   /* GREENLUT  (GREEN Lookup Table) */
    *bluelut=0x0000FB40;    /* BLUELUT  (BLUE Lookup Table) */
    *dp1_2=0x0000A5A5;      /* DP1_2  (Dithering Pattern duty 1/2) */
    *dp4_7=0x0BA5DA65;      /* DP4_7  (Dithering Pattern duty 4/7) */
    *dp3_5=0x000A5A5F;      /* DP3_5  (Dithering Pattern duty 3/5) */
    *dp2_3=0x00000D6B;      /* DP2_3  (Dithering Pattern duty 2/3) */
    *dp5_7=0x0EB7B5ED;      /* DP5_7  (Dithering Pattern duty 5/7) */
    *dp3_4=0x00007DBE;      /* DP3_4  (Dithering Pattern duty 3/4) */
    *dp4_5=0x0007EBDF;      /* DP4_5  (Dithering Pattern duty 4/5) */
    *dp6_7=0x07FDFBFE;      /* DP6_7  (Dithering Pattern duty 6/7) */
    *dithmode=0x00000000;   /* DITHMODE  (Dithering Mode) */

    /* Enable the panel now that the settings are in place */
    *lcdcon1=0x10C0DF41;    /* LCDCON1  (LCD Control 1) */
}

void draw_char(uint32_t x, uint32_t y, uint8_t c, uint8_t color)
{
    /* Draw a single ascii character on screen at location x,y */
    uint32_t i;
    uint32_t j;

    /* Chars are 7 pixels tall, 5 pixels wide */
    for (i=0; i<7; i++)
    {
        for (j=0; j<5; j++)
        {
            if (((ascii_lookup[c-32][i])<<(3+j)) & 0x80)
            {
                set_pixel(x+j, y+i, color);
            }
        }
    }
}

void draw_string(const uint8_t* s, uint32_t x, uint32_t y, uint8_t color)
{
    /* Draw a string of ascii chars on screen */
    while (*s)
    {
        draw_char(x, y, *s, color);
        x+=6;
        s++;
    }
}
